package com.gestionnaires;

import java.sql.*;
import java.util.Vector;

enum OperationBDD
{
	/**
	 * op&eacute;ration de s&eacute;lection .
	 */
	OP_BDD_SELECTION,
	/**
	 * op&eacute;ration de cr&eacute;ation de table.
	 */
	OP_BDD_CREER_TABLE,
	/**
	 * op&eacute;ration de suppression de table.
	 */
	OP_BDD_SUPPR_TABLE
}

public class GestionConnexion
{
	/*
	 * VARIABLES
	 */
	// paramtres
	private ParametresConnexion paramsConn;

	// connexion
	private Connection connection = null;
	private Statement statement = null;
	
	// requte SQL
	private String requeteSQL = "";
	private ResultSet resultSet = null;

	// erreurs - anomalies
	private Vector<String> vs_erreurs = null;
	private Vector<String> vs_anomalies = null;
	

	/*
	 * CONSTRUCTEUR
	 */
	
	public GestionConnexion()
		throws ClassNotFoundException, Exception
	{
		/*
		 * Init les autres variables
		 */
		this.statement = null;
		this.resultSet = null;
		this.requeteSQL = "";
		
		this.paramsConn = new ParametresConnexion();

		//init la gestion des erreurs - anomalies 
		this.vs_erreurs = new Vector<String>();
		this.vs_anomalies = new Vector<String>();
		
		try
		{
			// Rcupration du driver
			Class.forName(ParametresConnexion.CLASSE).newInstance();
		}
		catch (ClassNotFoundException ex)
		{
			//remonte l'exception
			throw new ClassNotFoundException("La classe '"+ParametresConnexion.CLASSE+"' n'a pas t trouve.");
		}
		catch (Exception ex)
		{
			//remonte l'exception
			throw ex;
		}
	}

	
	/*
	 * METHODES D'INSTANCES
	 */
	public void ouvrir(ParametresConnexion params)
	{
		//Vrifie le paramtre
		if (params == null)
			throw new NullPointerException();

		//Copie les paramtres
		this.paramsConn.CopierDepuis(params);
		
		//Appelle la mthode sans paramtres
		this.ouvrir();
	}

	public void ouvrir()
	{
		try
		{
			// ferme la conexion prcdente
			this.fermerConnexion();

			// init la nouvelle connexion
			this.connection = DriverManager.getConnection(this.paramsConn.Url,
					this.paramsConn.Utilisateur, this.paramsConn.Pwd);

			this.traiterWarning(this.connection.getWarnings());
		}
		catch (SQLException ex)
		{
			this.traiterSQLException(ex);

			this.fermerConnexion();
		}
	}

	public boolean estOuverte()
	{
		boolean bOuverte = false;

		try
		{
			if (this.connection != null)
				bOuverte = !this.connection.isClosed();
		}
		catch (SQLException ex)
		{
			this.traiterSQLException(ex);
		}

		return bOuverte;
	}

	public void fermerRessourcesRequete()
	{
		try
		{
			// Ferme le resultSet
			if (this.resultSet != null)
			{
				this.resultSet.close();
				this.resultSet = null;
			}
			
			// Ferme le statement
			if (this.statement != null)
			{
				this.statement.close();
				this.statement = null;
			}
			
			// Vide la requte
			this.requeteSQL = "";
		}
		catch (SQLException ex)
		{
			this.traiterSQLException(ex);
		}
	}

	public void fermerConnexion()
	{
		try
		{
			// Ferme le statement
			this.fermerRessourcesRequete();
			
			// Ferme la connexion
			if (this.connection != null)
			{
				this.connection.close();
				this.connection = null;
			}
		}
		catch (SQLException ex)
		{
			this.traiterSQLException(ex);
		}
	}

	public boolean lancerRequeteSelection(String query)
	{
		boolean res = true;

		try
		{
			// verifie la connexion
			if (this.connection != null)
			{
				// libre les ressources de la prcdente requte
				this.fermerRessourcesRequete();
				
				// prepare la requete
				this.statement = this.connection.createStatement();
				this.traiterWarning(this.statement.getWarnings());
				
				// emet la requete
				this.resultSet = this.statement.executeQuery(query);
				this.traiterWarning(this.resultSet.getWarnings());
				
				// si on est arriv jusqu'ici, c'est qu'il n'y a pas eu de leve d'exception
				// =>actualise la requte
				this.requeteSQL = query;
			}
		}
		catch (SQLException ex)
		{
			this.traiterSQLException(ex);

			// libre les ressources utilises
			this.fermerRessourcesRequete();
			
			//annonce l'chec de la requte SQL
			res = false;
		}

		return res;
	}

	public int lancerRequeteModifBDD(String query)
	{
		int res = 0;

		try
		{
			// verifie la connexion
			if (this.connection != null)
			{
				// libre les ressources de la prcdente requte
				this.fermerRessourcesRequete();
				
				// prepare la requete
				this.statement = this.connection.createStatement();
				
				// emet la requete
				res = this.statement.executeUpdate(query);
				this.traiterWarning(this.statement.getWarnings());
				
				// si on est arriv jusqu'ici, c'est qu'il n'y a pas eu de leve d'exception
				// =>actualise la requte
				this.requeteSQL = query;
			}
		}
		catch (SQLException ex)
		{
			this.traiterSQLException(ex);

			// libre les ressources utilises
			this.fermerRessourcesRequete();
			
			//annonce l'chec de la requte SQL
			res = -1;
		}

		return res;
	}

	public ParametresConnexion getParametresConnexion()
	{
		//Retourne une copie de l'objet contenant les paramtres de connexion
		return new ParametresConnexion(this.paramsConn);
	}

	public String getRequeteSQL()
	{
		return this.requeteSQL;
	}

	public Vector<String> getInfosConnexion()
	{
		Vector<String> vs_infosConn = new Vector<String>();
		DatabaseMetaData dmd;

		try
		{
			// Vrifie que la connexion soit ouverte
			if (this.estOuverte())
			{
				// Etat de la connexion
				vs_infosConn.add("Etat de la connexion : ouverte");
				vs_infosConn.add("------------------------------");
				
				// Obtient les meta-datas de la base
				dmd = this.connection.getMetaData();

				vs_infosConn.add("Connexion : " + dmd.getURL());
				vs_infosConn.add("Driver : " + dmd.getDriverName());
				vs_infosConn.add("Version : " + dmd.getDriverVersion());
			}
			else
			{
				// Etat de la connexion
				vs_infosConn.add("Etat de la connexion : ferme");
				vs_infosConn.add("------------------------------");

				vs_infosConn.add("Connexion : " + this.paramsConn.Url);//contient le Serveur, le Port et la Base
				vs_infosConn.add("Utilisateur : " + this.paramsConn.Utilisateur);
				vs_infosConn.add("Mot de passe : " + this.paramsConn.Pwd.length() + " caractre(s)");
			}
		}
		catch (SQLException ex)
		{
			// Erreur lors de l'obtention du nom des colonnes
			this.traiterSQLException(ex);
		}

		return vs_infosConn;
	}

	public Vector<String> getNomsColonnes()
	{
		Vector<String> vs_cols = new Vector<String>();
		ResultSetMetaData rsmd;

		try
		{
			// Vrifie si la requte a dj&agrave; t lance
			if (this.resultSet != null)
			{
				rsmd = this.resultSet.getMetaData(); // obtient les meta-datas de la requte
				int nbrCols = rsmd.getColumnCount();

				for (int idCol = 1; idCol <= nbrCols; idCol++)
					if (rsmd.isSearchable(idCol))
						vs_cols.add(rsmd.getColumnName(idCol));
			}
			else if (this.statement == null)
				// anomalie
				this.vs_anomalies.add("Vous n'avez pas encore lanc la requte!!");
		}
		catch (SQLException ex)
		{
			// Erreur lors de l'obtention du nom des colonnes
			this.traiterSQLException(ex);
		}

		return vs_cols;
	}

	public Vector<Vector<Object>> getDataVector()
	{
		Vector<Vector<Object>> v_datas = new Vector<Vector<Object>>();
		ResultSetMetaData rsmd;

		try
		{
			rsmd = this.resultSet.getMetaData();
			// Vrifie qu'il y ai un rsultat et place le curseur au dbut
			if (rsmd.getColumnCount() > 0)
			{
				while (this.resultSet.next())
				{
					Vector<Object> vo_ligne = new Vector<Object>();
					// cr la ligne
					for (int idCol = 1; idCol <= rsmd.getColumnCount(); idCol++)
						vo_ligne.add(this.resultSet.getObject(idCol));
					// ajoute la ligne au vecteur principal
					v_datas.add(vo_ligne);
				}
			}

			if (v_datas.isEmpty()) // anomalie
				this.vs_anomalies.add("Aucune donne trouve!!");
		}
		catch (SQLException ex)
		{
			// Probleme lors de l'obtention des donnees
			this.traiterSQLException(ex);
		}

		return v_datas;
	}

	/*
	 * GESTION DES ERREURS - ANOMALIES
	 */
	public void traiterSQLException(SQLException ex)
	{
		String str_ex;

		str_ex = "********* SQLException *********\n";
		while (ex != null)
		{
			str_ex = "SQLState : " + ex.getSQLState() + "\n";
			str_ex += "Message : " + ex.getMessage() + "\n";
			str_ex += "------------------------------\n";
			this.vs_erreurs.add(str_ex);

			ex = ex.getNextException();
		}
	}

	public void traiterWarning(SQLWarning warn)
	{
		String str_an;

		str_an = "********* SQLWarning *********\n";
		while (warn != null)
		{
			str_an = "SQLState : " + warn.getSQLState() + "\n";
			str_an += "Message : " + warn.getMessage() + "\n";
			str_an += "------------------------------\n";
			this.vs_anomalies.add(str_an);

			warn = warn.getNextWarning();
		}
	}

	public Vector<String> getErreurs()
	{
		return new Vector<String>(this.vs_erreurs);
	}

	public Vector<String> getAnomalies()
	{
		return new Vector<String>(this.vs_anomalies);
	}
	
	public void effaceErreurs()
	{
		this.vs_erreurs.clear();
	}

	public void effaceAnomalies()
	{
		this.vs_anomalies.clear();
	}
}
